@@ -1,22 +1,22 @@
 /* Sum -- efficiently sum a list of floating-point numbers
 
-Copyright 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -33,6 +33,8 @@ MA 02110-1301, USA. */
      const mpfr_s *const*     : no
      const mpfr_s **const     : no
      const mpfr_s *const*const: no
+   VL: this is not a bug, but a feature. See the reason here:
+     http://c-faq.com/ansi/constmismatch.html
 */
 static void heap_sort (mpfr_srcptr *const, unsigned long, mpfr_srcptr *);
 static void count_sort (mpfr_srcptr *const, unsigned long, mpfr_srcptr *,
@@ -239,7 +241,7 @@ mpfr_sum (mpfr_ptr ret, mpfr_ptr *const tab_p, unsigned long n, mp_rnd_t rnd)
 {
   mpfr_t cur_sum;
   mp_prec_t prec;
-  mpfr_srcptr *perm, *const tab = (mpfr_srcptr *const) tab_p;
+  mpfr_srcptr *perm, *const tab = (mpfr_srcptr *) tab_p;
   int k, error_trap;
   MPFR_ZIV_DECL (loop);
   MPFR_SAVE_EXPO_DECL (expo);
