@@ -1,22 +1,22 @@
 /* Generic test file for functions with one mpfr_t argument and an integer.
 
-Copyright 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -25,7 +25,7 @@ MA 02110-1301, USA. */
 # define INTEGER_TYPE mp_limb_t
 #endif
 #ifndef RAND_FUNCTION
-# define RAND_FUNCTION mpfr_random
+# define RAND_FUNCTION(x) mpfr_urandomb ((x), RANDS)
 #endif
 #ifndef INT_RAND_FUNCTION
 # define INT_RAND_FUNCTION() (INTEGER_TYPE) randlimb ()
@@ -65,7 +65,7 @@ test_generic_ui (mp_prec_t p0, mp_prec_t p1, unsigned int N)
               mpfr_set_exp (x, mpfr_get_emin ());
             }
           u = INT_RAND_FUNCTION ();
-          rnd = (mp_rnd_t) RND_RAND ();
+          rnd = RND_RAND ();
           mpfr_set_prec (y, yprec);
           compare = TEST_FUNCTION (y, x, u, rnd);
           if (mpfr_can_round (y, yprec, rnd, rnd, prec))
