@@ -1,22 +1,22 @@
 /* Test file for mpfr_get_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -27,18 +27,18 @@ MA 02110-1301, USA. */
 #include "mpfr-test.h"
 
 static void
-check3 (double d, mp_rnd_t rnd, char *res)
+check3 (char *d, mp_rnd_t rnd, char *res)
 {
   mpfr_t x;
   char *str;
   mp_exp_t e;
 
   mpfr_init2 (x, 53);
-  mpfr_set_d (x, d, rnd);
+  mpfr_set_str (x, d, 10, rnd);
   str = mpfr_get_str (NULL, &e, 10, 5, x, rnd);
   if (strcmp (str, res))
     {
-      printf ("Error in mpfr_get_str for x=%1.20e\n", d);
+      printf ("Error in mpfr_get_str for x=%s\n", d);
       printf ("got %s instead of %s\n", str, res);
       exit (1);
     }
@@ -765,6 +765,48 @@ check_small (void)
   s = mpfr_get_str (NULL, &e, 32, 9, x, GMP_RNDN);
   mpfr_free_str (s);
 
+  mpfr_set_prec (x, 7);
+  mpfr_set_str_binary (x, "0.1010101E10");
+  s = mpfr_get_str (NULL, &e, 10, 2, x, GMP_RNDU);
+  mpfr_free_str (s);
+
+  /* checks rounding of negative numbers */
+  mpfr_set_prec (x, 7);
+  mpfr_set_str (x, "-11.5", 10, GMP_RNDN);
+  s = mpfr_get_str (NULL, &e, 10, 2, x, GMP_RNDD);
+  if (strcmp (s, "-12"))
+    {
+      printf ("Error in mpfr_get_str for x=-11.5 and rnd=GMP_RNDD\n"
+              "got %s instead of -12\n", s);
+      exit (1);
+  }
+  mpfr_free_str (s);
+
+  s = mpfr_get_str (NULL, &e, 10, 2, x, GMP_RNDU);
+  if (strcmp (s, "-11"))
+    {
+      printf ("Error in mpfr_get_str for x=-11.5 and rnd=GMP_RNDU\n");
+      exit (1);
+    }
+  mpfr_free_str (s);
+
+  /* bug found by Jean-Pierre Merlet, produced error in mpfr_get_str */
+  mpfr_set_prec (x, 128);
+  mpfr_set_str_binary (x, "0.10111001100110011001100110011001100110011001100110011001100110011001100110011001100110011001100110011001100110011001100110011010E3");
+  s = mpfr_get_str (NULL, &e, 10, 0, x, GMP_RNDU);
+  mpfr_free_str (s);
+
+  mpfr_set_prec (x, 381);
+  mpfr_set_str_binary (x, "0.111111111111111111111111111111111111111111111111111111111111111111101110110000100110011101101101001010111000101111000100100011110101010110101110100000010100001000110100000100011111001000010010000010001010111001011110000001110010111101100001111000101101100000010110000101100100000101010110010110001010100111001111100011100101100000100100111001100010010011110011011010110000001000010");
+  s = mpfr_get_str (NULL, &e, 10, 0, x, GMP_RNDD);
+  if (e != 0)
+    {
+      printf ("Error in mpfr_get_str for x=0.999999..., exponent is %d"
+              " instead of 0\n", (int) e);
+      exit (1);
+    }
+  mpfr_free_str (s);
+
   mpfr_clear (x);
 }
 
@@ -1107,34 +1149,34 @@ main (int argc, char *argv[])
   for (i = 0; i < ITER; i++)
     {
       m = 2 + (randlimb () % (MAX_DIGITS - 1));
-      mpfr_random (x);
+      mpfr_urandomb (x, RANDS);
       e = (mp_exp_t) (randlimb () % 21) - 10;
       mpfr_set_exp (x, (e == -10) ? mpfr_get_emin () :
                     ((e == 10) ? mpfr_get_emax () : e));
       b = 2 + (randlimb () % 35);
-      r = (mp_rnd_t) RND_RAND();
+      r = RND_RAND ();
       mpfr_get_str (s, &f, b, m, x, r);
     }
   mpfr_clear (x);
 
   check_large ();
-  check3 (4.059650008e-83, GMP_RNDN, "40597");
-  check3 (-6.606499965302424244461355e233, GMP_RNDN, "-66065");
-  check3 (-7.4, GMP_RNDN, "-74000");
-  check3 (0.997, GMP_RNDN, "99700");
-  check3 (-4.53063926135729747564e-308, GMP_RNDN, "-45306");
-  check3 (2.14478198760196000000e+16, GMP_RNDN, "21448");
-  check3 (7.02293374921793516813e-84, GMP_RNDN, "70229");
+  check3 ("4.059650008e-83", GMP_RNDN, "40597");
+  check3 ("-6.606499965302424244461355e233", GMP_RNDN, "-66065");
+  check3 ("-7.4", GMP_RNDN, "-74000");
+  check3 ("0.997", GMP_RNDN, "99700");
+  check3 ("-4.53063926135729747564e-308", GMP_RNDN, "-45306");
+  check3 ("2.14478198760196000000e+16", GMP_RNDN, "21448");
+  check3 ("7.02293374921793516813e-84", GMP_RNDN, "70229");
 
-  check3 (-6.7274500420134077e-87, GMP_RNDN, "-67275");
-  check3 (-6.7274500420134077e-87, GMP_RNDZ, "-67274");
-  check3 (-6.7274500420134077e-87, GMP_RNDU, "-67274");
-  check3 (-6.7274500420134077e-87, GMP_RNDD, "-67275");
+  check3 ("-6.7274500420134077e-87", GMP_RNDN, "-67275");
+  check3 ("-6.7274500420134077e-87", GMP_RNDZ, "-67274");
+  check3 ("-6.7274500420134077e-87", GMP_RNDU, "-67274");
+  check3 ("-6.7274500420134077e-87", GMP_RNDD, "-67275");
 
-  check3 (6.7274500420134077e-87, GMP_RNDN, "67275");
-  check3 (6.7274500420134077e-87, GMP_RNDZ, "67274");
-  check3 (6.7274500420134077e-87, GMP_RNDU, "67275");
-  check3 (6.7274500420134077e-87, GMP_RNDD, "67274");
+  check3 ("6.7274500420134077e-87", GMP_RNDN, "67275");
+  check3 ("6.7274500420134077e-87", GMP_RNDZ, "67274");
+  check3 ("6.7274500420134077e-87", GMP_RNDU, "67275");
+  check3 ("6.7274500420134077e-87", GMP_RNDD, "67274");
 
   check_bug_base2k ();
   check_reduced_exprange ();
