@@ -1,22 +1,22 @@
 /* Test file for mpfr_sin_cos.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -50,7 +50,7 @@ check53 (const char *xs, const char *sin_xs, const char *cos_xs,
 {
   mpfr_t xx, s, c;
 
-  mpfr_inits2 (53, xx, s, c, (void *) 0);
+  mpfr_inits2 (53, xx, s, c, (mpfr_ptr) 0);
   mpfr_set_str1 (xx, xs); /* should be exact */
   mpfr_sin_cos (s, c, xx, rnd_mode);
   if (mpfr_cmp_str1 (s, sin_xs))
@@ -71,7 +71,7 @@ check53 (const char *xs, const char *sin_xs, const char *cos_xs,
       printf(", expected %s\n", cos_xs);
       exit (1);
     }
-  mpfr_clears (xx, s, c, (void *) 0);
+  mpfr_clears (xx, s, c, (mpfr_ptr) 0);
 }
 
 static void
@@ -79,7 +79,7 @@ check53sin (const char *xs, const char *sin_xs, mp_rnd_t rnd_mode)
 {
   mpfr_t xx, s, c;
 
-  mpfr_inits2 (53, xx, s, c, (void *) 0);
+  mpfr_inits2 (53, xx, s, c, (mpfr_ptr) 0);
   mpfr_set_str1 (xx, xs); /* should be exact */
   mpfr_sin_cos (s, c, xx, rnd_mode);
   if (mpfr_cmp_str1 (s, sin_xs))
@@ -91,7 +91,7 @@ check53sin (const char *xs, const char *sin_xs, mp_rnd_t rnd_mode)
       printf(", expected %s\n", sin_xs);
       exit (1);
     }
-  mpfr_clears (xx, s, c, (void *) 0);
+  mpfr_clears (xx, s, c, (mpfr_ptr) 0);
 }
 
 static void
@@ -99,7 +99,7 @@ check53cos (const char *xs, const char *cos_xs, mp_rnd_t rnd_mode)
 {
   mpfr_t xx, c, s;
 
-  mpfr_inits2 (53, xx, s, c, (void *) 0);
+  mpfr_inits2 (53, xx, s, c, (mpfr_ptr) 0);
   mpfr_set_str1 (xx, xs); /* should be exact */
   mpfr_sin_cos (s, c, xx, rnd_mode);
   if (mpfr_cmp_str1 (c, cos_xs))
@@ -111,7 +111,7 @@ check53cos (const char *xs, const char *cos_xs, mp_rnd_t rnd_mode)
       printf(", expected %s\n", cos_xs);
       exit (1);
     }
-  mpfr_clears (xx, s, c, (void *) 0);
+  mpfr_clears (xx, s, c, (mpfr_ptr) 0);
 }
 
 static void
@@ -189,18 +189,18 @@ overflowed_sin_cos0 (void)
           mpfr_set_si (x, 0, GMP_RNDN);
           mpfr_neg (x, x, GMP_RNDN);
           mpfr_clear_flags ();
-          inex = mpfr_sin_cos (x, y, x, rnd);
+          inex = mpfr_sin_cos (x, y, x, (mp_rnd_t) rnd);
           if (! mpfr_overflow_p ())
             {
               printf ("Error in overflowed_sin_cos0 (rnd = %s):\n"
                       "  The overflow flag is not set.\n",
-                      mpfr_print_rnd_mode (rnd));
+                      mpfr_print_rnd_mode ((mp_rnd_t) rnd));
               err = 1;
             }
           if (! (mpfr_zero_p (x) && MPFR_SIGN (x) < 0))
             {
               printf ("Error in overflowed_sin_cos0 (rnd = %s):\n"
-                      "  Got sin = ", mpfr_print_rnd_mode (rnd));
+                      "  Got sin = ", mpfr_print_rnd_mode ((mp_rnd_t) rnd));
               mpfr_print_binary (x);
               printf (" instead of -0.\n");
               err = 1;
@@ -211,13 +211,14 @@ overflowed_sin_cos0 (void)
                 {
                   printf ("Error in overflowed_sin_cos0 (rnd = %s):\n"
                           "  The inexact value must be non-zero.\n",
-                          mpfr_print_rnd_mode (rnd));
+                          mpfr_print_rnd_mode ((mp_rnd_t) rnd));
                   err = 1;
                 }
               if (! mpfr_equal_p (y, z))
                 {
                   printf ("Error in overflowed_sin_cos0 (rnd = %s):\n"
-                          "  Got cos = ", mpfr_print_rnd_mode (rnd));
+                          "  Got cos = ",
+                          mpfr_print_rnd_mode ((mp_rnd_t) rnd));
                   mpfr_print_binary (y);
                   printf (" instead of 0.11111111E%d.\n", emax);
                   err = 1;
@@ -229,13 +230,14 @@ overflowed_sin_cos0 (void)
                 {
                   printf ("Error in overflowed_sin_cos0 (rnd = %s):\n"
                           "  The inexact value must be non-zero.\n",
-                          mpfr_print_rnd_mode (rnd));
+                          mpfr_print_rnd_mode ((mp_rnd_t) rnd));
                   err = 1;
                 }
               if (! (mpfr_inf_p (y) && MPFR_SIGN (y) > 0))
                 {
                   printf ("Error in overflowed_sin_cos0 (rnd = %s):\n"
-                          "  Got cos = ", mpfr_print_rnd_mode (rnd));
+                          "  Got cos = ",
+                          mpfr_print_rnd_mode ((mp_rnd_t) rnd));
                   mpfr_print_binary (y);
                   printf (" instead of +Inf.\n");
                   err = 1;
@@ -258,7 +260,7 @@ tiny (void)
   mpfr_t x, s, c;
   int i, inex;
 
-  mpfr_inits2 (64, x, s, c, (void *) 0);
+  mpfr_inits2 (64, x, s, c, (mpfr_ptr) 0);
 
   for (i = -1; i <= 1; i += 2)
     {
@@ -270,7 +272,7 @@ tiny (void)
       MPFR_ASSERTN (!mpfr_nan_p (c) && mpfr_cmp_ui (c, 1) == 0);
     }
 
-  mpfr_clears (x, s, c, (void *) 0);
+  mpfr_clears (x, s, c, (mpfr_ptr) 0);
 }
 
 /* bug found in nightly tests */
